@@ -1,5 +1,5 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 import copy
 from trytond.model import ModelView, ModelSQL, fields
 from trytond.wizard import Wizard, StateView, Button, StateTransition
@@ -7,6 +7,7 @@ from trytond.report import Report
 from trytond.pyson import Eval, If
 from trytond.transaction import Transaction
 from trytond.pool import Pool, PoolMeta
+from trytond.backend.database import CursorInterface
 
 try:
     import pytz
@@ -15,6 +16,8 @@ except ImportError:
     TIMEZONES = []
 TIMEZONES += [(None, '')]
 
+CursorInterface.cache_keys.update({'company', 'employee'})
+
 __all__ = ['Company', 'Employee', 'UserEmployee', 'User', 'Property',
     'Sequence', 'SequenceStrict', 'Date', 'CompanyConfigStart',
     'CompanyConfig', 'CompanyReport', 'LetterReport']
@@ -61,6 +64,10 @@ class Employee(ModelSQL, ModelView):
     party = fields.Many2One('party.party', 'Party', required=True)
     company = fields.Many2One('company.company', 'Company', required=True)
 
+    @staticmethod
+    def default_company():
+        return Transaction().context.get('company')
+
     def get_rec_name(self, name):
         return self.party.rec_name
 
@@ -138,25 +145,20 @@ class User:
 
     @fields.depends('main_company')
     def on_change_main_company(self):
-        return {
-            'company': self.main_company.id if self.main_company else None,
-            'employee': None,
-            }
+        self.company = self.main_company
+        self.employee = None
 
     @fields.depends('company', 'employees')
     def on_change_company(self):
         Employee = Pool().get('company.employee')
-        result = {
-            'employee': None,
-            }
+        self.employee = None
         if self.company and self.employees:
             employees = Employee.search([
                     ('id', 'in', [e.id for e in self.employees]),
                     ('company', '=', self.company.id),
                     ])
             if employees:
-                result['employee'] = employees[0].id
-        return result
+                self.employee = employees[0]
 
     @classmethod
     def _get_preferences(cls, user, context_only=False):
@@ -271,7 +273,7 @@ class Property:
     @classmethod
     def search(cls, domain, offset=0, limit=None, order=None, count=False,
             query=False):
-        if Transaction().user == 0 and not 'user' in Transaction().context:
+        if Transaction().user == 0 and 'user' not in Transaction().context:
             domain = ['AND', domain[:], ('company', '=', None)]
         return super(Property, cls).search(domain, offset=offset, limit=limit,
             order=order, count=count, query=query)
@@ -325,7 +327,7 @@ class CompanyConfig(Wizard):
     start = StateView('company.company.config.start',
         'company.company_config_start_view_form', [
             Button('Cancel', 'end', 'tryton-cancel'),
-            Button('Ok', 'company', 'tryton-ok', True),
+            Button('OK', 'company', 'tryton-ok', True),
             ])
     company = StateView('company.company',
         'company.company_view_form', [
@@ -351,11 +353,10 @@ class CompanyConfig(Wizard):
 class CompanyReport(Report):
 
     @classmethod
-    def parse(cls, report, records, data, localcontext):
-        user = Pool().get('res.user')(Transaction().user)
-        localcontext['company'] = user.company
-        return super(CompanyReport, cls).parse(report, records, data,
-            localcontext)
+    def get_context(cls, records, data):
+        report_context = super(CompanyReport, cls).get_context(records, data)
+        report_context['company'] = report_context['user'].company
+        return report_context
 
 
 class LetterReport(CompanyReport):
